import getBoundingClientRect from "./getBoundingClientRect.js";
import getScrollSum from "./getScrollSum.js";
import getBorders from "./getBorders.js";
import { isHTMLElement } from "./instanceOf.js"; // offsets without `border`

function getInnerOffsets(offsetParent) {
  var rect = getBoundingClientRect(offsetParent);
  var borders = getBorders(offsetParent);
  return {
    x: rect.x + borders.left,
    y: rect.y + borders.top
  };
} // Returns the composite rect of an element relative to its offsetParent.
// Composite means it takes into account transforms as well as layout.


export default function getCompositeRect(elementOrVirtualElement, offsetParent, isFixed) {
  if (isFixed === void 0) {
    isFixed = false;
  }

  var rect = getBoundingClientRect(elementOrVirtualElement);
  var scrollSum = getScrollSum(isFixed ? [] : [offsetParent]);
  var offsets = isHTMLElement(offsetParent) && !isFixed ? getInnerOffsets(offsetParent) : {
    x: 0,
    y: 0
  };
  return {
    x: rect.left + scrollSum.scrollLeft - offsets.x,
    y: rect.top + scrollSum.scrollTop - offsets.y,
    width: rect.width,
    height: rect.height
  };
}